/*
 * Project name:
      DAC click (Driving external Digital-to-Analog converter)
 * Copyright:
     (c) Mikroelektronika, 2011.
 * Revision History:
     20111125:
       - initial release (FJ);
     20130625:
       - modified for EasyAVR v7 (MV);
 * Description:
     This is a sample program which demonstrates the use of the Microchip's
     MCP4921 12-bit D/A converter with AVR MCUs. This device accepts digital
     input (number from 0..4095) and transforms it to the output voltage,
     ranging from 0..Vref. In this example the DAC communicates with MCU
     through the SPI communication.
     Buttons PA0 and PA1 are used to change value sent to DAC.
 * Test configuration:

     MCU:             ATmega32
                      http://www.atmel.com/Images/doc2503.pdf
     Dev.Board:       EasyAVR v7
                      ac:EasyAVR_V7
                      http://www.mikroe.com/easyavr/
     Oscillator:      8.0000 MHz, 8.0000 MHz Crystal
     extern module:   DAC Click
                      ac:DAC_Click
                      http://www.mikroe.com/click/dac/
                      2x16 LCD character display
                      ac:LCD_2x16ch
                      http://www.mikroe.com/rs/store/components/
     SW:              mikroC PRO for AVR
                      http://www.mikroe.com/mikroc/avr/
 * NOTES:
 
     - Place DAC click board at the mikroBUS socket 1 on the EasyAVR7 board.
     - Put power supply jumper (J5) on the EasyAVR7 board in according to the
       PWR_SEL SMD jumper position.
     - Put Button Press Level dip switch PA to + position and PULL DOWN PA0 and 
       PA1
     - Place LCD in its socket and make sure you turned dip switch SW3.1 on
     - Turn on dip switches: SW5.4, SW5.5, SW5.6
 */

// DAC click module connections
sbit Chip_Select_Direction at DDA5_bit;
sbit Chip_Select at PORTA5_bit;
// End DAC click module connections

// Lcd module connections
sbit LCD_RS at PORTA2_bit;
sbit LCD_EN at PORTD6_bit;
sbit LCD_D4 at PORTC4_bit;
sbit LCD_D5 at PORTC5_bit;
sbit LCD_D6 at PORTC6_bit;
sbit LCD_D7 at PORTC7_bit;

sbit LCD_RS_Direction at DDA2_bit;
sbit LCD_EN_Direction at DDD6_bit;
sbit LCD_D4_Direction at DDC4_bit;
sbit LCD_D5_Direction at DDC5_bit;
sbit LCD_D6_Direction at DDC6_bit;
sbit LCD_D7_Direction at DDC7_bit;
// End Lcd module connections

// Variable declaration
unsigned int value;
unsigned int i;
unsigned char updateDAC;
char str[16];

// Lcd constants
char txt1[] = "mikroElektronika";
char txt2[] = "DAC click";
char txt3[] = "MCP4921  TEST";
char txt4[] = "output:";

// DAC increments (0..4095) --> output voltage (0..Vref)
void DAC_Output(unsigned int valueDAC) {
  char temp;

  Chip_Select = 0;                   // Select DAC chip

  // Send High Byte
  temp = (valueDAC >> 8) & 0x0F;     // Store valueDAC[11..8] to temp[3..0]
  temp |= 0x30;                      // Define DAC setting, see MCP4921 datasheet
  SPI1_Write(temp);                  // Send high byte via SPI

  // Send Low Byte
  temp = valueDAC;                   // Store valueDAC[7..0] to temp[7..0]
  SPI1_Write(temp);                  // Send low byte via SPI

  Chip_Select = 1;                   // Deselect DAC chip
}

// Update data on the Lcd display
void Update_Lcd(unsigned int valueDAC) {
  IntToStr(valueDAC, str);           // Convert DAC value to string
  Lcd_Out(2, 9, str);                // Display string on Lcd
}

void main(){

  // Configure DAC chip select line
  Chip_Select_Direction = 1;         // Set CS# pin as Output
  Chip_Select = 0;                   // Deselect DAC

  // Configure user buttons
  DDA0_bit = 0;                    // Set PA0 pin as input
  DDA1_bit = 0;                    // Set PA1 pin as input

  // Initialize SPI1 module
  SPI1_Init_Advanced(_SPI_MASTER, _SPI_FCY_DIV4, _SPI_CLK_LO_LEADING);

  Lcd_Init();                        // Initialize Lcd

  Lcd_Cmd(_LCD_CLEAR);               // Clear display
  Lcd_Cmd(_LCD_CURSOR_OFF);          // Cursor off
  Lcd_Out(1,1,txt1);                 // Write text in first row
  Lcd_Out(2,4,txt2);                 // Write text in second row

  Delay_ms(1000);                    // delay 2 sec

  Lcd_Cmd(_LCD_CLEAR);               // Clear display
  Lcd_Out(1,2,txt3);                 // Write text in first row
  Lcd_Out(2,2,txt4);                 // Write text in second row

  // Set inital DAC output value to the mid range
  value = 2048;

  while(1) {                                // Endless loop
    if ((PINA0_bit) && (value < 4095)) {      // If PA0 button is pressed
      value++;                              //   increment value
    }
    else {
      if ((PINA1_bit) && (value > 0)) {       // If PA1 button is pressed
        value--;                            //   decrement value
      }
    }
    DAC_Output(value);                      // Send value to DAC chip
    Update_Lcd(value);                      // Display value on Lcd
    Delay_ms(10);
    
  }
}